/**
 * chroma.js - JavaScript library for color conversions
 *
 * Copyright (c) 2011-2024, Gregor Aisch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name Gregor Aisch may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL GREGOR AISCH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -------------------------------------------------------
 *
 * chroma.js includes colors from colorbrewer2.org, which are released under
 * the following license:
 *
 * Copyright (c) 2002 Cynthia Brewer, Mark Harrower,
 * and The Pennsylvania State University.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * ------------------------------------------------------
 *
 * Named colors are taken from X11 Color Names.
 * http://www.w3.org/TR/css3-color/#svg-color
 *
 * @preserve
 */

!function(t,r){"object"==typeof exports&&"undefined"!=typeof module?module.exports=r():"function"==typeof define&&define.amd?define(r):(t="undefined"!=typeof globalThis?globalThis:t||self).chroma=r()}(this,(function(){"use strict";function t(t,r,n){return void 0===n&&(n=1),l(h(r,t),n)}for(var r={},n=0,e=["Boolean","Number","String","Function","Array","Date","RegExp","Undefined","Null"];n<e.length;n+=1){var o=e[n];r["[object "+o+"]"]=o.toLowerCase()}function a(t){return r[Object.prototype.toString.call(t)]||"object"}function i(t,r){return void 0===r&&(r=null),t.length>=3?Array.prototype.slice.call(t):"object"==a(t[0])&&r?r.split("").filter((function(r){return void 0!==t[0][r]})).map((function(r){return t[0][r]})):t[0].slice(0)}function s(t){if(t.length<2)return null;var r=t.length-1;return"string"==a(t[r])?t[r].toLowerCase():null}var u=Math.PI,l=Math.min,h=Math.max,c=u/180,f=180/u,d={format:{},autodetect:[]},g=function(){for(var r=[],n=arguments.length;n--;)r[n]=arguments[n];var e=this;if("object"===a(r[0])&&r[0].constructor&&r[0].constructor===this.constructor)return r[0];var o=s(r),i=!1;if(!o){i=!0,d.sorted||(d.autodetect=d.autodetect.sort((function(t,r){return r.p-t.p})),d.sorted=!0);for(var u=0,l=d.autodetect;u<l.length;u+=1){var h=l[u];if(o=h.test.apply(h,r))break}}if(!d.format[o])throw new Error("unknown format: "+r);var c=d.format[o].apply(null,i?r:r.slice(0,-1));e._rgb=function(r){r._clipped=!1,r._unclipped=r.slice(0);for(var n=0;n<=3;n++)n<3?((r[n]<0||r[n]>255)&&(r._clipped=!0),r[n]=t(r[n],0,255)):3===n&&(r[n]=t(r[n],0,1));return r}(c),3===e._rgb.length&&e._rgb.push(1)};g.prototype.toString=function(){return"function"==a(this.hex)?this.hex():"["+this._rgb.join(",")+"]"};var m=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(g,[null].concat(t)))};m.version="3.0.0-0";var p=function(t){return Math.round(100*t)/100},b=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n,e,o=(t=i(t,"rgba"))[0],a=t[1],s=t[2],u=l(o/=255,a/=255,s/=255),c=h(o,a,s),f=(c+u)/2;return c===u?(n=0,e=Number.NaN):n=f<.5?(c-u)/(c+u):(c-u)/(2-c-u),o==c?e=(a-s)/(c-u):a==c?e=2+(s-o)/(c-u):s==c&&(e=4+(o-a)/(c-u)),(e*=60)<0&&(e+=360),t.length>3&&void 0!==t[3]?[e,n,f,t[3]]:[e,n,f]},v=function(t){return Math.round(100*t)/100},y={Kn:18,labWhitePoint:"d65",Xn:.95047,Yn:1,Zn:1.08883,t0:.137931034,t1:.206896552,t2:.12841855,t3:.008856452,kE:216/24389,kKE:8,kK:24389/27,RefWhiteRGB:{X:.95047,Y:1,Z:1.08883},MtxRGB2XYZ:{m00:.4124564390896922,m01:.21267285140562253,m02:.0193338955823293,m10:.357576077643909,m11:.715152155287818,m12:.11919202588130297,m20:.18043748326639894,m21:.07217499330655958,m22:.9503040785363679},MtxXYZ2RGB:{m00:3.2404541621141045,m01:-.9692660305051868,m02:.055643430959114726,m10:-1.5371385127977166,m11:1.8760108454466942,m12:-.2040259135167538,m20:-.498531409556016,m21:.041556017530349834,m22:1.0572251882231791},As:.9414285350000001,Bs:1.040417467,Cs:1.089532651,MtxAdaptMa:{m00:.8951,m01:-.7502,m02:.0389,m10:.2664,m11:1.7135,m12:-.0685,m20:-.1614,m21:.0367,m22:1.0296},MtxAdaptMaI:{m00:.9869929054667123,m01:.43230526972339456,m02:-.008528664575177328,m10:-.14705425642099013,m11:.5183602715367776,m12:.04004282165408487,m20:.15996265166373125,m21:.0492912282128556,m22:.9684866957875502}},M=new Map([["a",[1.0985,.35585]],["b",[1.0985,.35585]],["c",[.98074,1.18232]],["d50",[.96422,.82521]],["d55",[.95682,.92149]],["d65",[.95047,1.08883]],["e",[1,1,1]],["f2",[.99186,.67393]],["f7",[.95041,1.08747]],["f11",[1.00962,.6435]],["icc",[.96422,.82521]]]);function w(t){var r=M.get(String(t).toLowerCase());if(!r)throw new Error("unknown Lab illuminant "+t);y.labWhitePoint=t,y.Xn=r[0],y.Zn=r[1]}function k(){return y.labWhitePoint}var x=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"rgb"),e=n[0],o=n[1],a=n[2],s=n.slice(3),u=j(e,o,a),l=function(t,r,n){var e=y.Xn,o=y.Yn,a=y.Zn,i=y.kE,s=y.kK,u=t/e,l=r/o,h=n/a,c=u>i?Math.pow(u,1/3):(s*u+16)/116,f=l>i?Math.pow(l,1/3):(s*l+16)/116,d=h>i?Math.pow(h,1/3):(s*h+16)/116;return[116*f-16,500*(c-f),200*(f-d)]}(u[0],u[1],u[2]);return[l[0],l[1],l[2]].concat(s.length>0&&s[0]<1?[s[0]]:[])};function _(t){var r=Math.sign(t);return((t=Math.abs(t))<=.04045?t/12.92:Math.pow((t+.055)/1.055,2.4))*r}var j=function(t,r,n){t=_(t/255),r=_(r/255),n=_(n/255);var e=y.MtxRGB2XYZ,o=y.MtxAdaptMa,a=y.MtxAdaptMaI,i=y.Xn,s=y.Yn,u=y.Zn,l=y.As,h=y.Bs,c=y.Cs,f=t*e.m00+r*e.m10+n*e.m20,d=t*e.m01+r*e.m11+n*e.m21,g=t*e.m02+r*e.m12+n*e.m22,m=i*o.m00+s*o.m10+u*o.m20,p=i*o.m01+s*o.m11+u*o.m21,b=i*o.m02+s*o.m12+u*o.m22,v=f*o.m00+d*o.m10+g*o.m20,M=f*o.m01+d*o.m11+g*o.m21,w=f*o.m02+d*o.m12+g*o.m22;return M*=p/h,w*=b/c,[f=(v*=m/l)*a.m00+M*a.m10+w*a.m20,d=v*a.m01+M*a.m11+w*a.m21,g=v*a.m02+M*a.m12+w*a.m22]},$=function(t){return Math.round(100*t)/100},A=Math.sqrt,F=Math.atan2,X=Math.round,Z=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"lab"),e=n[0],o=n[1],a=n[2],s=A(o*o+a*a),u=(F(a,o)*f+360)%360;return 0===X(1e4*s)&&(u=Number.NaN),[e,s,u]},E=Math.cbrt,Y=Math.pow,N=Math.sign,B=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"rgb"),e=n[0],o=n[1],a=n[2],s=n.slice(3),u=[C(e/255),C(o/255),C(a/255)],l=u[0],h=u[1],c=u[2],f=E(.4122214708*l+.5363325363*h+.0514459929*c),d=E(.2119034982*l+.6806995451*h+.1073969566*c),g=E(.0883024619*l+.2817188376*h+.6299787005*c);return[.2104542553*f+.793617785*d-.0040720468*g,1.9779984951*f-2.428592205*d+.4505937099*g,.0259040371*f+.7827717662*d-.808675766*g].concat(s.length>0&&s[0]<1?[s[0]]:[])};function C(t){var r=Math.abs(t);return r<.04045?t/12.92:(N(t)||1)*Y((r+.055)/1.055,2.4)}var O=function(t){return Math.round(100*t)/100},R=function(t){return Math.round(100*t)/100},L=Math.round,W=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"rgba"),e=s(t)||"rgb";if("hsl"===e.substr(0,3))return function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"hsla"),e=s(t)||"lsa";return n[0]=p(n[0]||0)+"deg",n[1]=p(100*n[1])+"%",n[2]=p(100*n[2])+"%","hsla"===e||n.length>3&&n[3]<1?(n[3]="/ "+(n.length>3?n[3]:1),e="hsla"):n.length=3,e.substr(0,3)+"("+n.join(" ")+")"}(b(n),e);if("lab"===e.substr(0,3)){var o=k();w("d50");var a=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"lab"),e=s(t)||"lab";return n[0]=v(n[0])+"%",n[1]=v(n[1]),n[2]=v(n[2]),"laba"===e||n.length>3&&n[3]<1?n[3]="/ "+(n.length>3?n[3]:1):n.length=3,"lab("+n.join(" ")+")"}(x(n),e);return w(o),a}if("lch"===e.substr(0,3)){var u=k();w("d50");var l=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"lch"),e=s(t)||"lab";return n[0]=$(n[0])+"%",n[1]=$(n[1]),n[2]=$(n[2])+"deg","lcha"===e||n.length>3&&n[3]<1?n[3]="/ "+(n.length>3?n[3]:1):n.length=3,"lch("+n.join(" ")+")"}(function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"rgb"),e=n[0],o=n[1],a=n[2],s=n.slice(3),u=x(e,o,a),l=u[0],h=u[1],c=u[2],f=Z(l,h,c);return[f[0],f[1],f[2]].concat(s.length>0&&s[0]<1?[s[0]]:[])}(n),e);return w(u),l}return"oklab"===e.substr(0,5)?function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"lab");return n[0]=O(100*n[0])+"%",n[1]=O(n[1]),n[2]=O(n[2]),n.length>3&&n[3]<1?n[3]="/ "+(n.length>3?n[3]:1):n.length=3,"oklab("+n.join(" ")+")"}(B(n)):"oklch"===e.substr(0,5)?function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"lab");return n[0]=R(100*n[0])+"%",n[1]=R(n[1]),n[2]=R(n[2])+"deg",n.length>3&&n[3]<1?n[3]="/ "+(n.length>3?n[3]:1):n.length=3,"oklch("+n.join(" ")+")"}(function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"rgb"),e=n[0],o=n[1],a=n[2],s=n.slice(3),u=B(e,o,a),l=u[0],h=u[1],c=u[2],f=Z(l,h,c);return[f[0],f[1],f[2]].concat(s.length>0&&s[0]<1?[s[0]]:[])}(n)):(n[0]=L(n[0]),n[1]=L(n[1]),n[2]=L(n[2]),("rgba"===e||n.length>3&&n[3]<1)&&(n[3]="/ "+(n.length>3?n[3]:1),e="rgba"),e.substr(0,3)+"("+n.slice(0,"rgb"===e?3:4).join(" ")+")")},K=function(){for(var t,r=[],n=arguments.length;n--;)r[n]=arguments[n];var e,o,a,s=(r=i(r,"hsl"))[0],u=r[1],l=r[2];if(0===u)e=o=a=255*l;else{var h=[0,0,0],c=[0,0,0],f=l<.5?l*(1+u):l+u-l*u,d=2*l-f,g=s/360;h[0]=g+1/3,h[1]=g,h[2]=g-1/3;for(var m=0;m<3;m++)h[m]<0&&(h[m]+=1),h[m]>1&&(h[m]-=1),6*h[m]<1?c[m]=d+6*(f-d)*h[m]:2*h[m]<1?c[m]=f:3*h[m]<2?c[m]=d+(f-d)*(2/3-h[m])*6:c[m]=d;e=(t=[255*c[0],255*c[1],255*c[2]])[0],o=t[1],a=t[2]}return r.length>3?[e,o,a,r[3]]:[e,o,a,1]},G=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=(t=i(t,"lab"))[0],e=t[1],o=t[2],a=I(n,e,o),s=a[0],u=a[1],l=a[2],h=S(s,u,l);return[h[0],h[1],h[2],t.length>3?t[3]:1]},I=function(t,r,n){var e=y.kE,o=y.kK,a=y.kKE,i=y.Xn,s=y.Yn,u=y.Zn,l=(t+16)/116,h=.002*r+l,c=l-.005*n,f=h*h*h,d=c*c*c;return[(f>e?f:(116*h-16)/o)*i,(t>a?Math.pow((t+16)/116,3):t/o)*s,(d>e?d:(116*c-16)/o)*u]},P=function(t){var r=Math.sign(t);return((t=Math.abs(t))<=.0031308?12.92*t:1.055*Math.pow(t,1/2.4)-.055)*r},S=function(t,r,n){var e=y.MtxAdaptMa,o=y.MtxAdaptMaI,a=y.MtxXYZ2RGB,i=y.RefWhiteRGB,s=y.Xn,u=y.Yn,l=y.Zn,h=s*e.m00+u*e.m10+l*e.m20,c=s*e.m01+u*e.m11+l*e.m21,f=s*e.m02+u*e.m12+l*e.m22,d=i.X*e.m00+i.Y*e.m10+i.Z*e.m20,g=i.X*e.m01+i.Y*e.m11+i.Z*e.m21,m=i.X*e.m02+i.Y*e.m12+i.Z*e.m22,p=(t*e.m00+r*e.m10+n*e.m20)*(d/h),b=(t*e.m01+r*e.m11+n*e.m21)*(g/c),v=(t*e.m02+r*e.m12+n*e.m22)*(m/f),M=p*o.m00+b*o.m10+v*o.m20,w=p*o.m01+b*o.m11+v*o.m21,k=p*o.m02+b*o.m12+v*o.m22;return[255*P(M*a.m00+w*a.m10+k*a.m20),255*P(M*a.m01+w*a.m11+k*a.m21),255*P(M*a.m02+w*a.m12+k*a.m22)]},q=Math.sin,T=Math.cos,D=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"lch"),e=n[0],o=n[1],a=n[2];return isNaN(a)&&(a=0),[e,T(a*=c)*o,q(a)*o]}((t=i(t,"lch"))[0],t[1],t[2]),e=n[0],o=n[1],a=n[2],s=G(e,o,a);return[s[0],s[1],s[2],t.length>3?t[3]:1]},U=/^rgb\(\s*(-?\d+) \s*(-?\d+)\s* \s*(-?\d+)\s*\)$/,z=/^rgb\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*\)$/,H=/^rgba?\(\s*(-?\d+) \s*(-?\d+)\s* \s*(-?\d+)\s*\/\s*([01]|[01]?\.\d+)\)$/,J=/^rgba\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*,\s*([01]|[01]?\.\d+)\)$/,Q=/^rgb\(\s*(-?\d+(?:\.\d+)?)% \s*(-?\d+(?:\.\d+)?)%\s* \s*(-?\d+(?:\.\d+)?)%\s*\)$/,V=/^rgb\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,tt=/^rgba?\(\s*(-?\d+(?:\.\d+)?)% \s*(-?\d+(?:\.\d+)?)%\s* \s*(-?\d+(?:\.\d+)?)%\s*\/\s*([01]|[01]?\.\d+)\)$/,rt=/^rgba\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,nt=/^hsl\(\s*(-?\d+(?:\.\d+)?)deg \s*(-?\d+(?:\.\d+)?)%\s* \s*(-?\d+(?:\.\d+)?)%\s*\)$/,et=/^hsl\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,ot=/^hsla?\(\s*(-?\d+(?:\.\d+)?)deg \s*(-?\d+(?:\.\d+)?)%\s* \s*(-?\d+(?:\.\d+)?)%\s*\/\s*([01]|[01]?\.\d+)\)$/,at=/^hsla\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,it=/^lab\(\s*(-?\d+(?:\.\d+)?%?) \s*(-?\d+(?:\.\d+)?%?) \s*(-?\d+(?:\.\d+)?%?)\s*(?:\/\s*(\d+(?:\.\d+)?))?\)?$/,st=/^lch\(\s*(-?\d+(?:\.\d+)?%?) \s*(?:(-?\d+(?:\.\d+)?%?)|none) \s*(-?\d+(?:\.\d+)?(?:deg)?|none)\s*(?:\/\s*(\d+(?:\.\d+)?))?\)?$/,ut=Math.round,lt=function(r){return r.map((function(r,n){return n<=2?t(ut(r),0,255):r}))},ht=function(t,r,n,e){return void 0===r&&(r=0),void 0===n&&(n=100),void 0===e&&(e=!1),t.endsWith("%")&&(t=parseFloat(t.substr(0,t.length-1))/100,t=e?r+.5*(t+1)*(n-r):r+t*(n-r)),+t},ct=function(t,r){return"none"===t?r:t},ft=function(t){var r;if(t=t.toLowerCase().trim(),d.format.named)try{return d.format.named(t)}catch(t){}if((r=t.match(U))||(r=t.match(z))){for(var n=r.slice(1,4),e=0;e<3;e++)n[e]=+n[e];return n[3]=1,n}if((r=t.match(H))||(r=t.match(J))){for(var o=r.slice(1,5),a=0;a<4;a++)o[a]=+o[a];return o}if((r=t.match(Q))||(r=t.match(V))){for(var i=r.slice(1,4),s=0;s<3;s++)i[s]=ut(2.55*i[s]);return i[3]=1,i}if((r=t.match(tt))||(r=t.match(rt))){for(var u=r.slice(1,5),l=0;l<3;l++)u[l]=ut(2.55*u[l]);return u[3]=+u[3],u}if((r=t.match(nt))||(r=t.match(et))){var h=r.slice(1,4);h[1]*=.01,h[2]*=.01;for(var c=K(h),f=0;f<3;f++)c[f]=ut(c[f]);return c[3]=1,c}if((r=t.match(ot))||(r=t.match(at))){var g=r.slice(1,4);g[1]*=.01,g[2]*=.01;for(var m=K(g),p=0;p<3;p++)m[p]=ut(m[p]);return m[3]=+r[4],m}if(r=t.match(it)){var b=r.slice(1,4);b[0]=ht(b[0],0,100),b[1]=ht(b[1],-125,125,!0),b[2]=ht(b[2],-125,125,!0);var v=lt(G(b));return v[3]=void 0!==r[4]?+r[4]:1,v}if(r=t.match(st)){var y=r.slice(1,4);y[0]=ht(y[0],0,100),y[1]=ht(ct(y[1],0),0,150,!1),y[2]=+ct(y[2].replace("deg",""),0);var M=lt(D(y));return M[3]=void 0!==r[4]?+r[4]:1,M}};ft.test=function(t){return U.test(t)||H.test(t)||Q.test(t)||tt.test(t)||nt.test(t)||ot.test(t)||it.test(t)||z.test(t)||J.test(t)||V.test(t)||rt.test(t)||et.test(t)||at.test(t)},g.prototype.css=function(t){return W(this._rgb,t)};var dt=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(g,[null].concat(t,["css"])))};m.css=dt,d.format.css=ft,d.autodetect.push({p:5,test:function(t){for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];if(!r.length&&"string"===a(t)&&ft.test(t))return"css"}});var gt=/^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/,mt=/^#?([A-Fa-f0-9]{8}|[A-Fa-f0-9]{4})$/,pt=Math.round;g.prototype.hex=function(t){return function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"rgba"),e=n[0],o=n[1],a=n[2],u=n[3],l=s(t)||"auto";void 0===u&&(u=1),"auto"===l&&(l=u<1?"rgba":"rgb");var h="000000"+((e=pt(e))<<16|(o=pt(o))<<8|(a=pt(a))).toString(16);h=h.substr(h.length-6);var c="0"+pt(255*u).toString(16);switch(c=c.substr(c.length-2),l.toLowerCase()){case"rgba":return"#"+h+c;case"argb":return"#"+c+h;default:return"#"+h}}(this._rgb,t)};var bt=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(g,[null].concat(t,["hex"])))};m.hex=bt,d.format.hex=function(t){if(t.match(gt)){4!==t.length&&7!==t.length||(t=t.substr(1)),3===t.length&&(t=(t=t.split(""))[0]+t[0]+t[1]+t[1]+t[2]+t[2]);var r=parseInt(t,16);return[r>>16,r>>8&255,255&r,1]}if(t.match(mt)){5!==t.length&&9!==t.length||(t=t.substr(1)),4===t.length&&(t=(t=t.split(""))[0]+t[0]+t[1]+t[1]+t[2]+t[2]+t[3]+t[3]);var n=parseInt(t,16);return[n>>24&255,n>>16&255,n>>8&255,Math.round((255&n)/255*100)/100]}throw new Error("unknown hex color: "+t)},d.autodetect.push({p:4,test:function(t){for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];if(!r.length&&"string"===a(t)&&[3,4,5,6,7,8,9].indexOf(t.length)>=0)return"hex"}}),g.prototype.hsl=function(){return b(this._rgb)};var vt=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(g,[null].concat(t,["hsl"])))};m.hsl=vt,d.format.hsl=K,d.autodetect.push({p:2,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===a(t=i(t,"hsl"))&&3===t.length)return"hsl"}}),g.prototype.lab=function(){return x(this._rgb)};var yt=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(g,[null].concat(t,["lab"])))};Object.assign(m,{lab:yt,getLabWhitePoint:k,setLabWhitePoint:w}),d.format.lab=G,d.autodetect.push({p:2,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===a(t=i(t,"lab"))&&3===t.length)return"lab"}});var Mt=Math.pow,wt=Math.sign;function kt(t){var r=Math.abs(t);return r>.0031308?(wt(t)||1)*(1.055*Mt(r,1/2.4)-.055):12.92*t}g.prototype.oklab=function(){return B(this._rgb)};var xt=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(g,[null].concat(t,["oklab"])))};Object.assign(m,{oklab:xt}),d.format.oklab=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=(t=i(t,"lab"))[0],e=t[1],o=t[2],a=Mt(n+.3963377774*e+.2158037573*o,3),s=Mt(n-.1055613458*e-.0638541728*o,3),u=Mt(n-.0894841775*e-1.291485548*o,3);return[255*kt(4.0767416621*a-3.3077115913*s+.2309699292*u),255*kt(-1.2684380046*a+2.6097574011*s-.3413193965*u),255*kt(-.0041960863*a-.7034186147*s+1.707614701*u),t.length>3?t[3]:1]},d.autodetect.push({p:2,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===a(t=i(t,"oklab"))&&3===t.length)return"oklab"}});var _t=Math.round;g.prototype.rgb=function(t){return void 0===t&&(t=!0),!1===t?this._rgb.slice(0,3):this._rgb.slice(0,3).map(_t)},g.prototype.rgba=function(t){return void 0===t&&(t=!0),this._rgb.slice(0,4).map((function(r,n){return n<3?!1===t?r:_t(r):r}))};var jt=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(g,[null].concat(t,["rgb"])))};Object.assign(m,{rgb:jt}),d.format.rgb=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=i(t,"rgba");return void 0===n[3]&&(n[3]=1),n},d.autodetect.push({p:3,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===a(t=i(t,"rgba"))&&(3===t.length||4===t.length&&"number"==a(t[3])&&t[3]>=0&&t[3]<=1))return"rgb"}}),g.prototype.alpha=function(t,r){return void 0===r&&(r=!1),void 0!==t&&"number"===a(t)?r?(this._rgb[3]=t,this):new g([this._rgb[0],this._rgb[1],this._rgb[2],t],"rgb"):this._rgb[3]},g.prototype.darken=function(t){void 0===t&&(t=1);var r=this.lab();return r[0]-=y.Kn*t,new g(r,"lab").alpha(this.alpha(),!0)},g.prototype.brighten=function(t){return void 0===t&&(t=1),this.darken(-t)},g.prototype.darker=g.prototype.darken,g.prototype.brighter=g.prototype.brighten,g.prototype.get=function(t){var r=t.split("."),n=r[0],e=r[1],o=this[n]();if(e){var a=n.indexOf(e)-("ok"===n.substr(0,2)?2:0);if(a>-1)return o[a];throw new Error("unknown channel "+e+" in mode "+n)}return o};var $t={};function At(t,r,n){void 0===n&&(n=.5);for(var e=[],o=arguments.length-3;o-- >0;)e[o]=arguments[o+3];var i=e[0]||"lrgb";if($t[i]||e.length||(i=Object.keys($t)[0]),!$t[i])throw new Error("interpolation mode "+i+" is not defined");return"object"!==a(t)&&(t=new g(t)),"object"!==a(r)&&(r=new g(r)),$t[i](t,r,n).alpha(t.alpha()+n*(r.alpha()-t.alpha()))}g.prototype.mix=g.prototype.interpolate=function(t,r){void 0===r&&(r=.5);for(var n=[],e=arguments.length-2;e-- >0;)n[e]=arguments[e+2];return At.apply(void 0,[this,t,r].concat(n))},g.prototype.set=function(t,r,n){void 0===n&&(n=!1);var e=t.split("."),o=e[0],i=e[1],s=this[o]();if(i){var u=o.indexOf(i)-("ok"===o.substr(0,2)?2:0);if(u>-1){if("string"==a(r))switch(r.charAt(0)){case"+":case"-":s[u]+=+r;break;case"*":s[u]*=+r.substr(1);break;case"/":s[u]/=+r.substr(1);break;default:s[u]=+r}else{if("number"!==a(r))throw new Error("unsupported value for Color.set");s[u]=r}var l=new g(s,o);return n?(this._rgb=l._rgb,this):l}throw new Error("unknown channel "+i+" in mode "+o)}return s},g.prototype.tint=function(t){void 0===t&&(t=.5);for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];return At.apply(void 0,[this,"white",t].concat(r))},g.prototype.shade=function(t){void 0===t&&(t=.5);for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];return At.apply(void 0,[this,"black",t].concat(r))};var Ft=Math.sqrt,Xt=Math.pow;$t.lrgb=function(t,r,n){var e=t._rgb,o=e[0],a=e[1],i=e[2],s=r._rgb,u=s[0],l=s[1],h=s[2];return new g(Ft(Xt(o,2)*(1-n)+Xt(u,2)*n),Ft(Xt(a,2)*(1-n)+Xt(l,2)*n),Ft(Xt(i,2)*(1-n)+Xt(h,2)*n),"rgb")};return $t.oklab=function(t,r,n){var e=t.oklab(),o=r.oklab();return new g(e[0]+n*(o[0]-e[0]),e[1]+n*(o[1]-e[1]),e[2]+n*(o[2]-e[2]),"oklab")},Object.assign(m,{Color:g,valid:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];try{return new(Function.prototype.bind.apply(g,[null].concat(t))),!0}catch(t){return!1}},css:dt,hex:bt,hsl:vt,lab:yt,oklab:xt,rgb:jt,mix:At,interpolate:At}),m}));
